@@ -1091,9 +1091,14 @@ static void add_string_date(const char *d)
 	putenv("TZ=UTC0");
 	t = mktime(&tm);
 	if(env)
-		setenv("TZ", env, 1);
+	{
+		char *c = xmalloc(strlen(env) + 3 + 1);	/* Extra space for TZ and = */
+		sprintf(c, "TZ=%s", env);
+		putenv(c);
+		xfree(c);
+	}
 	else
-		unsetenv("TZ");
+		putenv("TZ");
 
 	if(n != 6 || t == (time_t)(-1))
 	{
@@ -1175,7 +1180,7 @@ static char *expand_string(const char *s, rcsfile_t *rcs, revision_t *r, rev_t *
 
 	zap_string();
 
-	sprintf(nb, "%d", rcs->nbranches);
+	sprintf(nb, "%d", rcs->nbranches + rcs->nfolds);
 	sprintf(nr, "%d", rcs->nsrev);
 	for(; *s; s++)
 	{
@@ -1618,7 +1623,7 @@ static void draw_branch(gdImagePtr im, branch_t *b)
 			for(i = 0; i < b->nrevs; i++)
 			{
 				revision_t *r = b->revs[i];
-				gdImageSetStyle(im, line, r->stripped ? 4 : 1);
+				gdImageSetStyle(im, line, r->stripped > 0 ? 4 : 1);
 				gdImageLine(im, xx, r->y, r->cx+r->w, r->y, gdStyled);
 				for(sign = l = 1; l < conf.thick_lines; l++)
 				{
@@ -1648,7 +1653,7 @@ static void draw_branch(gdImagePtr im, branch_t *b)
 			for(i = 0; i < b->nrevs; i++)
 			{
 				revision_t *r = b->revs[i];
-				gdImageSetStyle(im, line, r->stripped ? 4 : 1);
+				gdImageSetStyle(im, line, r->stripped > 0 ? 4 : 1);
 				gdImageLine(im, xx, r->y, r->cx, r->y, gdStyled);
 				for(sign = l = 1; l < conf.thick_lines; l++)
 				{
@@ -1681,7 +1686,7 @@ static void draw_branch(gdImagePtr im, branch_t *b)
 			for(i = 0; i < b->nrevs; i++)
 			{
 				revision_t *r = b->revs[i];
-				gdImageSetStyle(im, line, r->stripped ? 4 : 1);
+				gdImageSetStyle(im, line, r->stripped > 0 ? 4 : 1);
 				gdImageLine(im, r->cx, yy, r->cx, r->y+r->h, gdStyled);
 				for(sign = l = 1; l < conf.thick_lines; l++)
 				{
@@ -1711,7 +1716,7 @@ static void draw_branch(gdImagePtr im, branch_t *b)
 			for(i = 0; i < b->nrevs; i++)
 			{
 				revision_t *r = b->revs[i];
-				gdImageSetStyle(im, line, r->stripped ? 4 : 1);
+				gdImageSetStyle(im, line, r->stripped > 0 ? 4 : 1);
 				gdImageLine(im, r->cx, yy, r->cx, r->y, gdStyled);
 				for(sign = l = 1; l < conf.thick_lines; l++)
 				{
@@ -2760,6 +2765,7 @@ static void fold_branch(rcsfile_t *rcs, revision_t *r)
 							&rcs->branches[j+1],
 							(rcs->nbranches - j - 1)*sizeof(rcs->branches[0]));
 						rcs->nbranches--;
+						rcs->nfolds++;
 						break;
 					}
 
@@ -2814,9 +2820,9 @@ static void make_layout(rcsfile_t *rcs)
 			{
 				if(!bp->revs[j]->ntags && bp->revs[j]->stripped >= 0 && !bp->revs[j]->nbranches)
 				{
+					bp->revs[j]->stripped = 1;
 					memmove(&bp->revs[j], &bp->revs[j+1], (bp->nrevs-j-1) * sizeof(bp->revs[0]));
 					bp->nrevs--;
-					bp->revs[j]->stripped = 1;
 					j--;
 				}
 			}
@@ -3596,7 +3602,7 @@ static const char usage_str[] =
 	"  -[0-9] <txt> Use <txt> for expansion\n"
 	;
 
-#define VERSION_STR	"1.5.1"
+#define VERSION_STR	"1.5.2"
 #define NOTICE_STR	"Copyright (c) 2001,2002,2003,2004 B.Stultiens"
 
 static void append_slash(char **path)
