@@ -51,7 +51,7 @@ typedef struct
 		char	**s;
 		color_t	*c;
 		double	*d;
-		int	val;	/* FIXME: Oops, sizeof(int) must be sizeof(void *) */
+		INTTYPE	val;
 	} confref;
 } keyword_t;
 
@@ -98,6 +98,8 @@ static keyword_t keywords[] = {
 	{ "branch_dupbox",	TYPE_BOOLEAN,	{ &conf.branch_dupbox } },
 	{ "branch_fold",	TYPE_BOOLEAN,	{ &conf.branch_fold } },
 	{ "branch_foldall",	TYPE_BOOLEAN,	{ &conf.branch_foldall } },
+	{ "branch_resort",	TYPE_BOOLEAN,	{ &conf.branch_resort } },
+	{ "branch_subtree",	TYPE_STRING,	{ &conf.branch_subtree } },
 	{ "upside_down",	TYPE_BOOLEAN,	{ &conf.upside_down } },
 	{ "left_right",		TYPE_BOOLEAN,	{ &conf.left_right } },
 	{ "auto_stretch",	TYPE_BOOLEAN,	{ &conf.auto_stretch } },
@@ -114,6 +116,11 @@ static keyword_t keywords[] = {
 	{ "parse_logs",		TYPE_BOOLEAN,	{ &conf.parse_logs } },
 	{ "html_level",		TYPE_NUMBER,	{ &conf.html_level } },
 	{ "thick_lines",	TYPE_NUMBER,	{ &conf.thick_lines } },
+	{ "msg_color",		TYPE_COLOR,	{ &conf.msg_color } },
+	{ "msg_font",		TYPE_FONT,	{ &conf.msg_font.gdfont } },
+	{ "msg_ttfont",		TYPE_STRING,	{ &conf.msg_font.ttfont } },
+	{ "msg_ttsize",		TYPE_DOUBLE,	{ &conf.msg_font.ttsize } },
+	{ "rev_hidenumber",	TYPE_BOOLEAN,	{ &conf.rev_hidenumber } },
 	{ "rev_color",		TYPE_COLOR,	{ &conf.rev_color } },
 	{ "rev_bgcolor",	TYPE_COLOR,	{ &conf.rev_bgcolor } },
 	{ "rev_font",		TYPE_FONT,	{ &conf.rev_font.gdfont } },
@@ -135,12 +142,22 @@ static keyword_t keywords[] = {
 	{ "merge_color",	TYPE_COLOR,	{ &conf.merge_color } },
 	{ "merge_from",		TYPE_STRING,	{ &conf.merge_from } },
 	{ "merge_to",		TYPE_STRING,	{ &conf.merge_to } },
+	{ "merge_findall",	TYPE_BOOLEAN,	{ &conf.merge_findall } },
 	{ "merge_front",	TYPE_BOOLEAN,	{ &conf.merge_front } },
 	{ "merge_nocase",	TYPE_BOOLEAN,	{ &conf.merge_nocase } },
+	{ "merge_arrows",	TYPE_BOOLEAN,	{ &conf.merge_arrows } },
+	{ "merge_cvsnt",	TYPE_BOOLEAN,	{ &conf.merge_cvsnt } },
+	{ "merge_cvsnt_color",	TYPE_COLOR,	{ &conf.merge_cvsnt_color } },
+	{ "arrow_width",	TYPE_NUMBER,	{ &conf.arrow_width } },
+	{ "arrow_length",	TYPE_NUMBER,	{ &conf.arrow_length } },
 	{ "tag_color",		TYPE_COLOR,	{ &conf.tag_color } },
 	{ "tag_font",		TYPE_FONT,	{ &conf.tag_font.gdfont } },
 	{ "tag_ttfont",		TYPE_STRING,	{ &conf.tag_font.ttfont } },
 	{ "tag_ttsize",		TYPE_DOUBLE,	{ &conf.tag_font.ttsize } },
+	{ "tag_ignore",		TYPE_STRING,	{ &conf.tag_ignore } },
+	{ "tag_ignore_merge",	TYPE_BOOLEAN,	{ &conf.tag_ignore_merge } },
+	{ "tag_nocase",		TYPE_BOOLEAN,	{ &conf.tag_nocase } },
+	{ "tag_negate",		TYPE_BOOLEAN,	{ &conf.tag_negate } },
 	{ "title",		TYPE_STRING,	{ &conf.title } },
 	{ "title_x",		TYPE_NUMBER,	{ &conf.title_x } },
 	{ "title_y",		TYPE_NUMBER,	{ &conf.title_y } },
@@ -155,6 +172,8 @@ static keyword_t keywords[] = {
 	{ "margin_right",	TYPE_NUMBER,	{ &conf.margin_right } },
 	{ "image_type",		TYPE_NUMBER,	{ &conf.image_type } },
 	{ "image_quality",	TYPE_NUMBER,	{ &conf.image_quality } },
+	{ "image_compress",	TYPE_NUMBER,	{ &conf.image_compress } },
+	{ "image_interlace",	TYPE_BOOLEAN,	{ &conf.image_interlace } },
 	{ "map_name",		TYPE_STRING,	{ &conf.map_name } },
 	{ "map_branch_href",	TYPE_STRING,	{ &conf.map_branch_href } },
 	{ "map_branch_alt",	TYPE_STRING,	{ &conf.map_branch_alt } },
@@ -162,6 +181,8 @@ static keyword_t keywords[] = {
 	{ "map_rev_alt",	TYPE_STRING,	{ &conf.map_rev_alt } },
 	{ "map_diff_href",	TYPE_STRING,	{ &conf.map_diff_href } },
 	{ "map_diff_alt",	TYPE_STRING,	{ &conf.map_diff_alt } },
+	{ "map_merge_href",	TYPE_STRING,	{ &conf.map_merge_href } },
+	{ "map_merge_alt",	TYPE_STRING,	{ &conf.map_merge_alt } },
 	{ "jpeg",		TYPE_VALUE,	{ (void *)IMAGE_JPEG } },
 	{ "png",		TYPE_VALUE,	{ (void *)IMAGE_PNG } },
 	{ "gif",		TYPE_VALUE,	{ (void *)IMAGE_GIF } },
@@ -529,7 +550,7 @@ static int config_lex(void)
 						c[1] = get_input();
 						c[2] = '\0';
 						if(!isxdigit((int)(unsigned char)c[0]) || !isxdigit((int)(unsigned char)c[1]))
-							yyerror("Invalid hex escape");
+							stack_msg(MSG_ERR, "config: %d: Invalid hex escape", line_number);
 						add_str((int)strtol(c, NULL, 16));
 						break;
 					case '0':
@@ -548,8 +569,8 @@ static int config_lex(void)
 						add_str((int)strtol(c, NULL, 8));
 						break;
 					case EOF:
-						  yyerror("Unexpected EOF in escape");
-						  break;
+						yyerror("Unexpected EOF in escape");
+						break;
 					}
 					break;
 				case '"':
@@ -590,11 +611,15 @@ static int config_lex(void)
 				skw.keyword = get_str();
 				kw = bsearch(&skw, keywords, NKEYWORDS, sizeof(keywords[0]), cmp_kw);
 				if(!kw)
-					yyerror("Unknown keyword '%s'", skw.keyword);
+				{
+					stack_msg(MSG_ERR, "config: %d: Unknown keyword '%s'", line_number, skw.keyword);
+					yylval.kw = NULL;
+					return TYPE_KEYWORD;
+				}
 				xfree(skw.keyword);
 				if(kw->type == TYPE_VALUE)
 				{
-					yylval.i = kw->confref.val;
+					yylval.i = (int)kw->confref.val;
 					return TYPE_NUMBER;
 				}
 				yylval.kw = kw;
@@ -626,13 +651,13 @@ static int config_lex(void)
 				{
 					yylval.d = strtod(s, &eptr);
 					if(*eptr)
-						yyerror("Invalid floating point number");
+						stack_msg(MSG_ERR, "config: %d: Invalid floating point number", line_number);
 				}
 				else
 				{
 					yylval.i = strtol(s, &eptr, 0);
 					if(*eptr)
-						yyerror("Invalid number");
+						stack_msg(MSG_ERR, "config: %d: Invalid number", line_number);
 				}
 				xfree(s);
 				return type;
@@ -650,7 +675,8 @@ static void set_color(color_t *c, char *s)
 	if(*s != '#' || strlen(s) != 7)
 	{
 colorerror:
-		yyerror("Invalid color value");
+		stack_msg(MSG_ERR, "config: %d: Invalid color value '%s'", line_number, s);
+		return;
 	}
 	c->b = strtol(s+5, &cptr, 16);
 	if(*cptr)
@@ -708,7 +734,7 @@ static void config_parse(void)
 		if(token == EOF)
 		{
 			if(state)
-				yyerror("Unexpected EOF");
+				stack_msg(MSG_ERR, "config: %d: Unexpected EOF", line_number);
 			break;
 		}
 
@@ -721,14 +747,34 @@ static void config_parse(void)
 				state = 1;
 			}
 			else if(token != ';')
-				yyerror("Keyword expected");
+				stack_msg(MSG_ERR, "config: %d: Keyword expected", line_number);
 			break;
 		case 1:
 			if(token != '=')
-				yyerror("'=' expected");
-			state = 2;
+			{
+				stack_msg(MSG_ERR, "config: %d: '=' expected", line_number);
+				while(1)
+				{
+					token = config_lex();
+					if(token == ';')
+					{
+						state = 0;
+						break;
+					}
+					else if(token == EOF)
+						break;
+				}
+			}
+			else
+				state = 2;
 			break;
 		case 2:
+			if(!kw)
+			{
+				/* Error recovery of failed keyword */
+				state = 3;
+				break;
+			}
 			if(kw->type == TYPE_FONT || kw->type == TYPE_BOOLEAN)
 				t = TYPE_NUMBER;
 			else if(kw->type == TYPE_COLOR)
@@ -748,15 +794,15 @@ static void config_parse(void)
 				char *e;
 				switch(kw->type)
 				{
-				case TYPE_STRING:	e = "String"; break;
-				case TYPE_NUMBER:	e = "Number"; break;
-				case TYPE_COLOR:	e = "Color"; break;
-				case TYPE_FONT:		e = "Font"; break;
-				case TYPE_BOOLEAN:	e = "Boolean"; break;
-				case TYPE_DOUBLE:	e = "Double"; break;
-				default:		e = "Internal error: Unknown type"; break;
+				case TYPE_STRING:	e = "String"; yylval.str = xstrdup("error recovery"); break;
+				case TYPE_NUMBER:	e = "Number"; yylval.i = 0; break;
+				case TYPE_COLOR:	e = "Color"; yylval.str = xstrdup("#123456"); break;
+				case TYPE_FONT:		e = "Font"; yylval.i = 0; break;
+				case TYPE_BOOLEAN:	e = "Boolean"; yylval.i = 0; break;
+				case TYPE_DOUBLE:	e = "Double"; yylval.d = 0.0; break;
+				default:		e = "Internal error: Unknown type"; yylval.i = 0; break;
 				}
-				yyerror("%s expected", e);
+				stack_msg(MSG_ERR, "config: %d: %s expected", line_number, e);
 			}
 #ifdef DEBUG
 			printf("processing: '%s'\n", kw->keyword);
@@ -792,7 +838,7 @@ static void config_parse(void)
 			break;
 		case 3:
 			if(token != ';')
-				yyerror("';' expected");
+				stack_msg(MSG_ERR, "config: %d: ';' expected", line_number);
 			state = 0;
 			break;
 		default:
